@@ -13,45 +13,83 @@ _LOGGER = logging.getLogger(__name__)
 
 CHANGE_TIME_CACHE_DEFAULT = 5  # Default 5s
 
-_LEASES_CMD = 'cat {}/dnsmasq.leases'
+_LEASES_CMD = "cat {}/dnsmasq.leases"
 _LEASES_REGEX = re.compile(
-    r'\w+\s' +
-    r'(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2})))\s' +
-    r'(?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3})\s' +
-    r'(?P<host>([^\s]+))')
+    r"\w+\s"
+    r"(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2})))\s"
+    r"(?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3})\s"
+    r"(?P<host>([^\s]+))"
+)
 
 # Command to get both 5GHz and 2.4GHz clients
-_WL_CMD = 'for dev in `nvram get wl1_vifs && nvram get wl0_vifs && ' \
-          'nvram get wl_ifnames`; do ' \
-          'if type wlanconfig > /dev/null; then ' \
-          'wlanconfig $dev list | awk \'FNR > 1 {print substr($1, 0, 18)}\';' \
-          ' else wl -i $dev assoclist; fi; done'
-_WL_REGEX = re.compile(
-    r'\w+\s' +
-    r'(?P<mac>(([0-9A-F]{2}[:-]){5}([0-9A-F]{2})))')
+_WL_CMD = (
+    "for dev in `nvram get wl1_vifs && nvram get wl0_vifs && "
+    "nvram get wl_ifnames`; do "
+    "if type wlanconfig > /dev/null; then "
+    "wlanconfig $dev list | awk 'FNR > 1 {print substr($1, 0, 18)}';"
+    " else wl -i $dev assoclist; fi; done"
+)
+_WL_REGEX = re.compile(r"\w+\s" r"(?P<mac>(([0-9A-F]{2}[:-]){5}([0-9A-F]{2})))")
 
-_IP_NEIGH_CMD = 'ip neigh'
+_NVRAM_CMD = "nvram show"
+
+_IP_NEIGH_CMD = "ip neigh"
 _IP_NEIGH_REGEX = re.compile(
-    r'(?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3}|'
-    r'([0-9a-fA-F]{1,4}:){1,7}[0-9a-fA-F]{0,4}(:[0-9a-fA-F]{1,4}){1,7})\s'
-    r'\w+\s'
-    r'\w+\s'
-    r'(\w+\s(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2}))))?\s'
-    r'\s?(router)?'
-    r'\s?(nud)?'
-    r'(?P<status>(\w+))')
+    r"(?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3}|"
+    r"([0-9a-fA-F]{1,4}:){1,7}[0-9a-fA-F]{0,4}(:[0-9a-fA-F]{1,4}){1,7})\s"
+    r"\w+\s"
+    r"\w+\s"
+    r"(\w+\s(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2}))))?\s"
+    r"\s?(router)?"
+    r"\s?(nud)?"
+    r"(?P<status>(\w+))"
+)
 
-_ARP_CMD = 'arp -n'
+_ARP_CMD = "arp -n"
 _ARP_REGEX = re.compile(
-    r'.+\s' +
-    r'\((?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3})\)\s' +
-    r'.+\s' +
-    r'(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2})))' +
-    r'\s' +
-    r'.*')
+    r".+\s"
+    r"\((?P<ip>([0-9]{1,3}[\.]){3}[0-9]{1,3})\)\s"
+    r".+\s"
+    r"(?P<mac>(([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2})))"
+    r"\s"
+    r".*"
+)
 
-_RX_COMMAND = 'cat /sys/class/net/{}/statistics/rx_bytes'
-_TX_COMMAND = 'cat /sys/class/net/{}/statistics/tx_bytes'
+_RX_COMMAND = "cat /sys/class/net/{}/statistics/rx_bytes"
+_TX_COMMAND = "cat /sys/class/net/{}/statistics/tx_bytes"
+
+_MEMINFO_CMD = "cat /proc/meminfo"
+_LOADAVG_CMD = "cat /proc/loadavg"
+_ADDHOST_CMD = (
+    'cat /etc/hosts | grep -q "{ipaddress} {hostname}" || '
+    '(echo "{ipaddress} {hostname}" >> /etc/hosts && '
+    "kill -HUP `cat /var/run/dnsmasq.pid`)"
+)
+
+_NETDEV_CMD = "cat /proc/net/dev"
+_NETDEV_FIELDS = [
+    "tx_bytes",
+    "tx_packets",
+    "tx_errs",
+    "tx_drop",
+    "tx_fifo",
+    "tx_frame",
+    "tx_compressed",
+    "tx_multicast",
+    "rx_bytes",
+    "rx_packets",
+    "rx_errs",
+    "rx_drop",
+    "rx_fifo",
+    "rx_colls",
+    "rx_carrier",
+    "rx_compressed",
+]
+
+_TEMP_CMD = (
+    "wl -i eth1 phy_tempsense ; wl -i eth2 phy_tempsense ;"
+    " head -c20 /proc/dmu/temperature"
+)
 
 GET_LIST = {
     "DHCP": [
@@ -60,7 +98,7 @@ GET_LIST = {
         "dhcp_enable_x",
         "dhcp_start",
         "dhcp_end",
-        "dhcp_lease"
+        "dhcp_lease",
     ],
     "MODEL": ["model"],
     "QOS": [
@@ -85,13 +123,9 @@ GET_LIST = {
         "qos_sched",
         "qos_sticky",
         "qos_syn",
-        "qos_type"
-    ],
-    "REBOOT": [
-        "reboot_schedule",
-        "reboot_schedule_enable",
-        "reboot_time"
+        "qos_type",
     ],
+    "REBOOT": ["reboot_schedule", "reboot_schedule_enable", "reboot_time"],
     "WLAN": [
         "wan_dns",
         "wan_domain",
@@ -102,56 +136,46 @@ GET_LIST = {
         "wan_lease",
         "wan_mtu",
         "wan_realip_ip",
-        "wan_realip_state"
+        "wan_realip_state",
     ],
     "2G_GUEST_1": [
         "wl0.1_bss_enabled",
         "wl0.1_lanaccess",
         "wl0.1_ssid",
-        "wl0.1_wpa_psk"
+        "wl0.1_wpa_psk",
     ],
     "2G_GUEST_2": [
         "wl0.2_bss_enabled",
         "wl0.2_lanaccess",
         "wl0.2_ssid",
-        "wl0.2_wpa_psk"
+        "wl0.2_wpa_psk",
     ],
     "2G_GUEST_3": [
         "wl0.3_bss_enabled",
         "wl0.3_lanaccess",
         "wl0.3_ssid",
-        "wl0.3_wpa_psk"
-    ],
-    "2G_WIFI": [
-        "wl0_bss_enabled",
-        "wl0_chanspec",
-        "wl0_ssid",
-        "wl0_wpa_psk"
+        "wl0.3_wpa_psk",
     ],
+    "2G_WIFI": ["wl0_bss_enabled", "wl0_chanspec", "wl0_ssid", "wl0_wpa_psk"],
     "5G_GUEST_1": [
         "wl1.1_bss_enabled",
         "wl1.1_lanaccess",
         "wl1.1_ssid",
-        "wl1.1_wpa_psk"
+        "wl1.1_wpa_psk",
     ],
     "5G_GUEST_2": [
         "wl1.2_bss_enabled",
         "wl1.2_lanaccess",
         "wl1.2_ssid",
-        "wl1.2_wpa_psk"
+        "wl1.2_wpa_psk",
     ],
     "5G_GUEST_3": [
         "wl1.3_bss_enabled",
         "wl1.3_lanaccess",
         "wl1.3_ssid",
-        "wl1.3_wpa_psk"
-    ],
-    "5G_WIFI": [
-        "wl1_bss_enabled",
-        "wl1_chanspec",
-        "wl1_ssid",
-        "wl1_wpa_psk"
+        "wl1.3_wpa_psk",
     ],
+    "5G_WIFI": ["wl1_bss_enabled", "wl1_chanspec", "wl1_ssid", "wl1_wpa_psk"],
     "FIRMWARE": [
         "buildinfo",
         "buildno",
@@ -169,11 +193,11 @@ GET_LIST = {
         "webs_state_odm",
         "webs_state_update",
         "webs_state_upgrade",
-        "webs_state_url"
-    ]
+        "webs_state_url",
+    ],
 }
 
-Device = namedtuple('Device', ['mac', 'ip', 'name'])
+Device = namedtuple("Device", ["mac", "ip", "name"])
 
 
 async def _parse_lines(lines, regex):
@@ -197,10 +221,20 @@ async def _parse_lines(lines, regex):
 class AsusWrt:
     """This is the interface class."""
 
-    def __init__(self, host, port=None, use_telnet=False, username=None,
-                 password=None, ssh_key=None, mode='router', require_ip=False,
-                 time_cache=CHANGE_TIME_CACHE_DEFAULT, interface='eth0',
-                 dnsmasq='/var/lib/misc'):
+    def __init__(
+        self,
+        host,
+        port=None,
+        use_telnet=False,
+        username=None,
+        password=None,
+        ssh_key=None,
+        mode="router",
+        require_ip=False,
+        time_cache=CHANGE_TIME_CACHE_DEFAULT,
+        interface="eth0",
+        dnsmasq="/var/lib/misc",
+    ):
         """Init function."""
         self.require_ip = require_ip
         self.mode = mode
@@ -217,17 +251,16 @@ class AsusWrt:
         self.dnsmasq = dnsmasq
 
         if use_telnet:
-            self.connection = TelnetConnection(
-                host, port, username, password)
+            self.connection = TelnetConnection(host, port, username, password)
         else:
-            self.connection = SshConnection(
-                host, port, username, password, ssh_key)
+            self.connection = SshConnection(host, port, username, password, ssh_key)
 
-    async def async_get_nvram(self, toGet):
+    async def async_get_nvram(self, to_get):
+        """Gets nvram"""
         data = {}
-        if toGet in GET_LIST:
-            lines = await self.connection.async_run_command('nvram show')
-            for item in GET_LIST[toGet]:
+        if to_get in GET_LIST:
+            lines = await self.connection.async_run_command(_NVRAM_CMD)
+            for item in GET_LIST[to_get]:
                 regex = rf"{item}=([\w.\-/: ]+)"
                 for line in lines:
                     result = re.findall(regex, line)
@@ -237,62 +270,67 @@ class AsusWrt:
         return data
 
     async def async_get_wl(self):
+        """gets wl"""
         lines = await self.connection.async_run_command(_WL_CMD)
         if not lines:
             return {}
         result = await _parse_lines(lines, _WL_REGEX)
         devices = {}
         for device in result:
-            mac = device['mac'].upper()
+            mac = device["mac"].upper()
             devices[mac] = Device(mac, None, None)
         return devices
 
     async def async_get_leases(self, cur_devices):
+        """Gets leases"""
         lines = await self.connection.async_run_command(
-            _LEASES_CMD.format(self.dnsmasq))
+            _LEASES_CMD.format(self.dnsmasq)
+        )
         if not lines:
             return {}
-        lines = [line for line in lines if not line.startswith('duid ')]
+        lines = [line for line in lines if not line.startswith("duid ")]
         result = await _parse_lines(lines, _LEASES_REGEX)
         devices = {}
         for device in result:
             # For leases where the client doesn't set a hostname, ensure it
             # is blank and not '*', which breaks entity_id down the line.
-            host = device['host']
-            if host == '*':
-                host = ''
-            mac = device['mac'].upper()
+            host = device["host"]
+            if host == "*":
+                host = ""
+            mac = device["mac"].upper()
             if mac in cur_devices:
-                devices[mac] = Device(mac, device['ip'], host)
+                devices[mac] = Device(mac, device["ip"], host)
         return devices
 
     async def async_get_neigh(self, cur_devices):
+        """Gets neigh"""
         lines = await self.connection.async_run_command(_IP_NEIGH_CMD)
         if not lines:
             return {}
         result = await _parse_lines(lines, _IP_NEIGH_REGEX)
         devices = {}
         for device in result:
-            status = device['status']
-            if status is None or status.upper() != 'REACHABLE':
+            status = device["status"]
+            if status is None or status.upper() != "REACHABLE":
                 continue
-            if device['mac'] is not None:
-                mac = device['mac'].upper()
+            if device["mac"] is not None:
+                mac = device["mac"].upper()
                 old_device = cur_devices.get(mac)
                 old_ip = old_device.ip if old_device else None
-                devices[mac] = Device(mac, device.get('ip', old_ip), None)
+                devices[mac] = Device(mac, device.get("ip", old_ip), None)
         return devices
 
     async def async_get_arp(self):
+        """Gets arp"""
         lines = await self.connection.async_run_command(_ARP_CMD)
         if not lines:
             return {}
         result = await _parse_lines(lines, _ARP_REGEX)
         devices = {}
         for device in result:
-            if device['mac'] is not None:
-                mac = device['mac'].upper()
-                devices[mac] = Device(mac, device['ip'], None)
+            if device["mac"] is not None:
+                mac = device["mac"].upper()
+                devices[mac] = Device(mac, device["ip"], None)
         return devices
 
     async def async_get_connected_devices(self, use_cache=True):
@@ -302,8 +340,11 @@ class AsusWrt:
         responses. Some commands will not work on some routers.
         """
         now = datetime.utcnow()
-        if use_cache and self._dev_cache_timer and self._cache_time > \
-                (now - self._dev_cache_timer).total_seconds():
+        if (
+            use_cache
+            and self._dev_cache_timer
+            and self._cache_time > (now - self._dev_cache_timer).total_seconds()
+        ):
             return self._devices_cache
 
         devices = {}
@@ -313,7 +354,7 @@ class AsusWrt:
         devices.update(dev)
         dev = await self.async_get_neigh(devices)
         devices.update(dev)
-        if not self.mode == 'ap':
+        if not self.mode == "ap":
             dev = await self.async_get_leases(devices)
             devices.update(dev)
 
@@ -329,8 +370,11 @@ class AsusWrt:
     async def async_get_bytes_total(self, use_cache=True):
         """Retrieve total bytes (rx an tx) from ASUSWRT."""
         now = datetime.utcnow()
-        if use_cache and self._trans_cache_timer and self._cache_time > \
-                (now - self._trans_cache_timer).total_seconds():
+        if (
+            use_cache
+            and self._trans_cache_timer
+            and self._cache_time > (now - self._trans_cache_timer).total_seconds()
+        ):
             return self._transfer_rates_cache
 
         rx = await self.async_get_rx()
@@ -340,14 +384,16 @@ class AsusWrt:
     async def async_get_rx(self):
         """Get current RX total given in bytes."""
         data = await self.connection.async_run_command(
-            _RX_COMMAND.format(self.interface))
-        return float(data[0]) if data[0] != '' else None
+            _RX_COMMAND.format(self.interface)
+        )
+        return float(data[0]) if data[0] != "" else None
 
     async def async_get_tx(self):
         """Get current RX total given in bytes."""
         data = await self.connection.async_run_command(
-            _TX_COMMAND.format(self.interface))
-        return float(data[0]) if data[0] != '' else None
+            _TX_COMMAND.format(self.interface)
+        )
+        return float(data[0]) if data[0] != "" else None
 
     async def async_get_current_transfer_rates(self, use_cache=True):
         """Gets current transfer rates calculated in per second in bytes."""
@@ -378,16 +424,73 @@ class AsusWrt:
 
         self._latest_transfer_data = (
             math.ceil(rx / time_diff.total_seconds()) if rx > 0 else 0,
-            math.ceil(tx / time_diff.total_seconds()) if tx > 0 else 0)
+            math.ceil(tx / time_diff.total_seconds()) if tx > 0 else 0,
+        )
         return self._latest_transfer_data
 
-    async def async_current_transfer_human_readable(
-            self, use_cache=True):
+    async def async_current_transfer_human_readable(self, use_cache=True):
         """Gets current transfer rates in a human readable format."""
         rx, tx = await self.async_get_current_transfer_rates(use_cache)
 
         return "%s/s" % convert_size(rx), "%s/s" % convert_size(tx)
 
+    async def async_get_loadavg(self):
+        """Get loadavg."""
+        loadavg = list(
+            map(
+                lambda avg: float(avg),
+                (await self.connection.async_run_command(_LOADAVG_CMD))[0].split(" ")[
+                    0:3
+                ],
+            )
+        )
+        return loadavg
+
+    #    async def async_get_meminfo(self):
+    #        """Get Memory information."""
+    #        memory_info = await self.connection.async_run_command(_MEMINFO_CMD)
+    #        memory_info = filter(lambda s: s != '', memory_info)
+    #        ret = {}
+    #        for item in list(map(lambda i: i.split(' '), memory_info)):
+    #            name = re.sub(r'(?<!^)(?=[A-Z])', '_', item[0]).lower()
+    #            ret[name] = list(filter(lambda i: i != '', item[1].split(' ')))
+    #            ret[name][0] = int(ret[name][0])
+    #
+    #        return ret
+
+    async def async_add_dns_record(self, hostname, ipaddress):
+        """Add record to /etc/hosts and HUP dnsmask to catch this record."""
+        return await self.connection.async_run_command(
+            _ADDHOST_CMD.format(hostname=hostname, ipaddress=ipaddress)
+        )
+
+    async def async_get_interfaces_counts(self):
+        """Get counters for all network interfaces."""
+        lines = await self.connection.async_run_command(_NETDEV_CMD)
+        lines = list(
+            map(lambda i: list(filter(lambda j: j != "", i.split(" "))), lines[2:-1])
+        )
+        interfaces = map(
+            lambda i: [
+                i[0][0:-1],
+                dict(zip(_NETDEV_FIELDS, map(lambda j: int(j), i[1:]))),
+            ],
+            lines,
+        )
+        return dict(interfaces)
+
+    async def async_get_temperature(self):
+        """Get temperature for 2.4GHz/5.0GHz/CPU."""
+        [r24, r50, cpu] = map(
+            lambda l: l.split(" "), await self.connection.async_run_command(_TEMP_CMD)
+        )
+        [r24, r50, cpu] = [
+            float(r24[0]) / 2 + 20,
+            float(r50[0]) / 2 + 20,
+            float(cpu[2]),
+        ]
+        return {"2.4GHz": r24, "5.0GHz": r50, "CPU": cpu}
+
     @property
     def is_connected(self):
         return self.connection.is_connected
